(*
    ############################################################################

    Q[Ĵ߂̕V~[V

    t^A xNgZ

    2003/08/13 쐬

    ############################################################################
*)

unit iVector;

interface

uses
    Classes, SysUtils, Math;

const TOL: double = 0.0001;     // e덷

type
    TVector = class
    private

    public
        x           : double;
        y           : double;
        z           : double;

        constructor Create(xi, yi, zi: double); overload;
        constructor Create(); overload;

        function    Magnitude(): double;            // XJIȑ傫
        procedure   Normalize();                    // K(PʃxNg)
        procedure   Reverse();                      // ]

        function    Plus(u: TVector): TVector;      // ZĎԋp
        function    Minus(u: TVector): TVector;     // ZĎԋp
        function    Multiply(s: double): TVector;   // XJςĎԋp
        function    Divide(s: double): TVector;     // XJZĎԋp

        function    Conjugation(): TVector;         // xNgĕԋp
    end;

function    Plus(u, v: TVector): TVector;                       // xNg̉Z
function    Minus(u, v: TVector): TVector;                      // xNǧZ
function    Outer(u, v: TVector): TVector;                      // xNg̊O
function    Inner(u, v: TVector): double;                       // xNg̓
function    Multiply(u: TVector; s: double): TVector; overload; // XJZ
function    Multiply(s: double; u: TVector): TVector; overload; // XJZ
function    Divide(u: TVector; s: double): TVector;             // XJZ
function    TripleProduct(u, v, w: TVector): double;            // Od

implementation


// ********************************************************
//
//  TVector
//
// ********************************************************

// ========================================================
constructor TVector.Create(xi, yi, zi: double);
begin
    x := xi;
    y := yi;
    z := zi;
end;

// ========================================================
constructor TVector.Create();
begin
    Create(0, 0, 0);
end;

// ========================================================
function    TVector.Magnitude(): double;
begin
    Result := Sqrt(x * x + y * y + z * z);
end;

// ========================================================
procedure   TVector.Normalize();
var
    m: double;
begin
    m := Magnitude();
    if m <= TOL then m := 1;
    x := x / m;
    y := y / m;
    z := z / m;
    if abs(x) < TOL then x := 0;
    if abs(y) < TOL then y := 0;
    if abs(z) < TOL then z := 0;
end;

// ========================================================
procedure   TVector.Reverse();
begin
    x := - x;
    y := - y;
    z := - z;
end;

// ========================================================
function    TVector.Plus(u: TVector): TVector;
begin
    x := x + u.x;
    y := y + u.y;
    z := z + u.z;
    Result := Self;
end;

// ========================================================
function    TVector.Minus(u: TVector): TVector;
begin
    x := x - u.x;
    y := y - u.y;
    z := z - u.z;
    Result := Self;
end;

// ========================================================
function    TVector.Multiply(s: double): TVector;
begin
    x := x * s;
    y := y * s;
    z := z * s;
    Result := Self;
end;

// ========================================================
function    TVector.Divide(s: double): TVector;
begin
    x := x / s;
    y := y / s;
    z := z / s;
    Result := Self;
end;

// ========================================================
function    TVector.Conjugation(): TVector;
begin
    Result := TVector.Create(-x, -y, -z);
end;


// ********************************************************
//
//  Utility
//
// ********************************************************

// ========================================================
function    Plus(u, v: TVector): TVector;
var
    x, y, z: double;
begin
    x := u.x + v.x;
    y := u.y + v.y;
    z := u.z + v.z;
    Result := TVector.Create(x, y, z);
end;

// ========================================================
function    Minus(u, v: TVector): TVector;
var
    x, y, z: double;
begin
    x := u.x - v.x;
    y := u.y - v.y;
    z := u.z - v.z;
    Result := TVector.Create(x, y, z);
end;

// ========================================================
function    Outer(u, v: TVector): TVector;
var
    x, y, z: double;
begin
    x :=   u.y * v.z - u.z * v.y;
    y := - u.x * v.z + u.z * v.x;
    z :=   u.x * v.y - u.y * v.x;
    Result := TVector.Create(x, y, z);
end;

// ========================================================
function    Inner(u, v: TVector): double;
begin
    Result := u.x * v.x + u.y * v.y + u.z * v.z;
end;

// ========================================================
function    Multiply(u: TVector; s: double): TVector;
var
    x, y, z: double;
begin
    x := u.x * s;
    y := u.y * s;
    z := u.z * s;
    Result := TVector.Create(x, y, z);
end;

// ========================================================
function    Multiply(s: double; u: TVector): TVector;
begin
    Result := Multiply(u, s);
end;

// ========================================================
function    Divide(u: TVector; s: double): TVector;
var
    x, y, z: double;
begin
    x := u.x / s;
    y := u.y / s;
    z := u.z / s;
    Result := TVector.Create(x, y, z);
end;

// ========================================================
function    TripleProduct(u, v, w: TVector): double;
var
    x, y, z: double;
begin
    x := u.x * (  v.y * w.z - v.z * w.y);
    y := u.y * (- v.x * w.z + v.z * w.x);
    z := u.z * (  v.x * w.y - v.y * w.x);
    Result := x + y + z;
end;


end.

